/* ----------------------------------------------------------------------------
 *
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 *
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdm.h is required in all files using the RDM Database */
#include "rdm.h"

/* core07_db.h was generated from the DDL file core07_db.ddl by the ddlp.
 * utility.
 */
#include "core07_db.h"

/* core07_db_dbd.h was generated from the DDL file core07_db.ddl by the 
 * ddlp utility.  It is required to use the d_open_ptr API
 */
#include "core07_db_dbd.h"

/* EXAMPLE - Core for C */
/* Each time you run this example:
 *   1) The database is initialized (all existing contents removed)
 *   2) Several new student records are added to the database
 *   3) Several new course records are added to the database
 *   4) Students are registered for various courses
 *   5) A list of all the students and the courses they are registered for is
 *      displayed
 *   6) A list of all the courses and the students thare are registered for 
 *      them is displayed
 *
 * The DDL (Database Definition Language) is in core07_db.ddl. The database
 * schema is embedded into the application and the database will be created 
 * the first time the application is run.
 *
 * The following RDM Functions are illustrated:
 *      d_opentask/d_closetask
 *      d_open_ptr/d_close
 *      d_initialize
 *      d_fillnew/d_makenew
 *      d_setor
 *      d_connect
 *      d_recfrst/d_recnext
 *      d_findfm/d_findnm
 *      d_findco
 *      d_recread
 *      d_errorname
 *      d_errorstr
 *
 * They are fully documented in http://docs.raima.com/rdm/.
 *
 * Because the database is opened in exclusive mode lock and transactions
 * are not required.  Future core examples will use shared mode and will
 * demonstrate the use of transactions and locks.
 *
 * Each of these functions returns an integer status code, where the
 * value S_OKAY indicates a successful call.
 *
 * For simplicity, this example does not check all return codes, but good
 * programming practices would dictate that they are checked after each
 * RDM call.
 *
 * The actual database will be stored in a directory named 'core07_db' in the
 * project directory.
 */

int32_t register_for_course(
    const char *student_name,
    const char *course_id,
    DB_TASK    *task,
    int32_t     dbn)
{
    /* Look up the student and set it up as the owner of the ATTENDS set*/
    d_keyfind(STUDENT_NAME, student_name, task, dbn);
    d_setor(ATTENDS, task, dbn);

    /* Look up the course and set it up as the owner of the ATTENDED_BY set */
    d_keyfind(COURSE_ID, course_id, task, dbn);
    d_setor(ATTENDED_BY, task, dbn);

    /* create a new course_registration record */
    d_makenew(COURSE_REGISTRATION, task, dbn);

    /* Associate the student and the course via the course_registartion sets */
    d_connect(ATTENDS, task, dbn);
    d_connect(ATTENDED_BY, task, dbn);

    return S_OKAY;
}

/* =======================================================================
*/
int32_t main(void)
{
    DB_TASK        *task;
    int32_t         status;
    struct student  student_rec;
    struct course   course_rec;

    /* create the context for the database operations */
    d_opentask(&task);

    /* open the database for exclusive access, this will create the database
     * dictionary (core07_db.dbd file if it does not already exist.
     */
    status = d_open_ptr("core07_db", "x", &core07_db_dbd, sizeof(core07_db_dbd), task);
    if (status != S_OKAY)
    {
        fprintf (stderr, "Exclusive open failed, error = %s: %s\n", d_errorname(status), d_errorstr(status) );
        d_closetask (task);
        return (1);
    }

    /* Initializing the database will create the data files if they do not exist.
     * If the the data files already exist they will be removed and empty
     * files will be created.
     */
    d_initialize(task, CURR_DB);
    
    /* Create a few student records */
    strcpy(student_rec.name, "Jeff");
    d_fillnew(STUDENT, &student_rec, task, CURR_DB);

    strcpy(student_rec.name, "Brooke");
    d_fillnew(STUDENT, &student_rec, task, CURR_DB);

    strcpy(student_rec.name, "Jonah");
    d_fillnew(STUDENT, &student_rec, task, CURR_DB);

    strcpy(student_rec.name, "Norah");
    d_fillnew(STUDENT, &student_rec, task, CURR_DB);

    strcpy(student_rec.name, "Micah");
    d_fillnew(STUDENT, &student_rec, task, CURR_DB);

    /* A few classes */
    strcpy(course_rec.id, "ACCTG1A");
    strcpy(course_rec.title, "Principles of Accounting");
    d_fillnew(COURSE, &course_rec, task, CURR_DB);

    strcpy(course_rec.id, "MATH037");
    strcpy(course_rec.title, "Finite Mathematics");
    d_fillnew(COURSE, &course_rec, task, CURR_DB);

    strcpy(course_rec.id, "CAOTO15");
    strcpy(course_rec.title, "Business Communications");
    d_fillnew(COURSE, &course_rec, task, CURR_DB);

    strcpy(course_rec.id, "CBIS36");
    strcpy(course_rec.title, "Systems Analysis and Design");
    d_fillnew(COURSE, &course_rec, task, CURR_DB);

    strcpy(course_rec.id, "IBUS1");
    strcpy(course_rec.title, "Introduction to International Business");
    d_fillnew(COURSE, &course_rec, task, CURR_DB);

    /* Register students for some courses */
    register_for_course("Jeff", "IBUS1", task, CURR_DB);
    register_for_course("Jeff", "CAOTO15", task, CURR_DB);
    register_for_course("Jeff", "ACCTG1A", task, CURR_DB);
    register_for_course("Brooke", "CBIS36", task, CURR_DB);
    register_for_course("Brooke", "IBUS1", task, CURR_DB);
    register_for_course("Jonah", "MATH037", task, CURR_DB);
    register_for_course("Jonah", "CBIS36", task, CURR_DB);
    register_for_course("Norah", "IBUS1", task, CURR_DB);
    register_for_course("Norah", "ACCTG1A", task, CURR_DB);
    register_for_course("Norah", "MATH037", task, CURR_DB);
    register_for_course("Micah", "ACCTG1A", task, CURR_DB);

    /* Loop through each student record and diplay the class records it
       is associated with
     */
    printf("\nList of courses each student is registered for\n");
    for (status = d_recfrst (STUDENT, task, CURR_DB);
         status == S_OKAY;
         status = d_recnext (task, CURR_DB))
    {
        /* Establish the current student instance as the owner of the 
         * ATTENDS set
         */
        d_setor(ATTENDS, task, CURR_DB);

        /* Read and display the current student record */
        d_recread(&student_rec, task, CURR_DB);
        printf("%s\n", student_rec.name);

        /* Find all of the course_registrations records related to this student */
        for (status = d_findfm (ATTENDS, task, CURR_DB);
             status == S_OKAY;
             status = d_findnm (ATTENDS, task, CURR_DB))
        {
            /* Find and read the course instance related to this course_registration */
            d_findco(ATTENDED_BY, task, CURR_DB);
            status = d_recread(&course_rec, task, CURR_DB);
            printf("\t%s: %s\n", course_rec.id, course_rec.title);
        }
        printf("\n");
    }

    /* Loop through each course record and diplay the students that have
       registered for it
     */
    printf("\nList of students registered for each course\n");
    for (status = d_recfrst (COURSE, task, CURR_DB);
         status == S_OKAY;
         status = d_recnext (task, CURR_DB))
    {
        /* Establish the current course record as the owner of the 
         * ATTENDED_BY set
         */
        d_setor(ATTENDED_BY, task, CURR_DB);

        /* Read and display the current MainRec2 instance */
        d_recread(&course_rec, task, CURR_DB);
        printf("%s: %s\n", course_rec.id, course_rec.title);

        /* Find all of the course_registration records related to this course */
        for (status = d_findfm (ATTENDED_BY, task, CURR_DB);
             status == S_OKAY;
             status = d_findnm (ATTENDED_BY, task, CURR_DB))
        {
            /* Find and read the student record related to this 
             * course_registration
             */
            d_findco(ATTENDS, task, CURR_DB);
            d_recread(&student_rec, task, CURR_DB);
            printf("\t%s\n", student_rec.name);
        }
        printf("\n");
    }
    
    /* close the database */
    d_close(task);
    d_closetask(task);
    
    printf("\nPress <Enter> to continue . . .\n");
    getchar();

    return EXIT_SUCCESS;
}
